"""Views."""

from typing import Optional

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.models import Permission
from django.shortcuts import get_object_or_404, redirect, render
from django.utils.translation import gettext_lazy as _
from django.views.decorators.http import require_POST

from allianceauth.authentication.models import User
from allianceauth.notifications.models import Notification
from allianceauth.services.hooks import get_extension_logger

from .models import Clocking, Eviction

logger = get_extension_logger(__name__)


def add_common_context(request, context: Optional[dict] = None) -> dict:
    """adds the common context used by all view"""
    context = context or {}
    new_context = {
        **{
            "user_has_manager_permission": request.user.has_perm(
                "evictionsplit.manager"
            ),
        },
        **context,
    }
    return new_context


@login_required
@permission_required("evictionsplit.basic_access")
def index(request):
    """Module index"""
    user = request.user
    logger.info("Displaying index for user %s", user)
    user_active_evictions = user.evictions_participating.filter(active=True)
    for eviction in user_active_evictions:
        eviction.is_standby = eviction.is_user_on_standby(user)
        eviction.is_doorstop = eviction.is_user_on_doorstop(user)
        eviction.time_standby = eviction.user_standby_time(user)
        eviction.time_doorstop = eviction.user_doorstop_time(user)

    return render(
        request,
        "evictionsplit/index.html",
        add_common_context(
            request,
            {
                "user_active_evictions": user_active_evictions,
            },
        ),
    )


@login_required
@permission_required("evictionsplit.basic_access")
def list_evictions(request):
    """Lists all active evictions that can be applied to join"""
    logger.info("Displaying evictions for user %s", request.user)

    active_evictions = Eviction.objects.filter(active=True)
    user_active_evictions = request.user.evictions_participating.filter(active=True)

    return render(
        request,
        "evictionsplit/list.html",
        add_common_context(
            request,
            {
                "active_evictions": active_evictions,
                "user_active_evictions": user_active_evictions,
            },
        ),
    )


@login_required
@permission_required("evictionsplit.manager")
@require_POST
def new_eviction(request):
    """Creates a new eviction from the GUI"""
    eviction_name = request.POST.get("name")
    logger.debug("Creating eviction called %s", eviction_name)
    eviction = Eviction.create(eviction_name, request.user)
    eviction.save()

    logger.info("Successfully created eviction %s", eviction.name)

    return redirect("evictionsplit:list")


def notify_managers_of_applicant(applicant: User, eviction: Eviction):
    """Sends a notification to every manager that a new application needs to be processed"""
    logger.info("Sending notification to all managers of applicant id %s", applicant.id)
    manager_permission = Permission.objects.get(
        name="Can create and manage evictions", codename="manager"
    )
    manager_users = manager_permission.user_set.all()

    for manager in manager_users:
        logger.debug("Sendint notification to manager id %s", manager.id)
        Notification.objects.notify_user(
            manager,
            title=_("New application to eviction "),
            message=_(
                "%(applicant)s applied to eviction %(eviction)s and needs to be processed"
            )
            % {
                "applicant": applicant.profile.main_character.character_name,
                "eviction": eviction.name,
            },
            level=Notification.Level.INFO,
        )


@login_required
@permission_required("evictionsplit.basic_access")
def apply(request, eviction_id):
    """Allows a user to apply to the evicton"""
    logger.info("User id %s applying to eviction id %s", request.user.id, eviction_id)
    eviction = get_object_or_404(Eviction, id=eviction_id)
    eviction.user_apply(request.user)

    messages.success(request, _("Successfully applied to eviction %s") % eviction.name)

    # TODO move this logic in models
    if request.user.has_perm("evictionsplit.manager"):
        logger.info(
            "Accepting the applycation automatically since user id %s is a manager",
            request.user.id,
        )
        eviction.validate_apply(request.user)
        messages.success(request, _("Automatically validated the application."))

    else:
        notify_managers_of_applicant(request.user, eviction)

    return redirect("evictionsplit:list")


@login_required
@permission_required("evictionsplit.manager")
def accept_application(request, eviction_id, user_id):
    """Validates a user application"""
    logger.info(
        "User id %s accepting the application of user id %s to eviction id %s",
        request.user.id,
        user_id,
        eviction_id,
    )
    eviction = get_object_or_404(Eviction, id=eviction_id)
    user = get_object_or_404(User, id=user_id)
    eviction.validate_apply(user)

    Notification.objects.notify_user(
        user=user,
        title=_("Eviction application accepted"),
        message=_("Your application to %s was accepted") % eviction.name,
        level=Notification.Level.INFO,
    )

    messages.success(
        request,
        _("Successfully added %(user)s to %(eviction)s.")
        % {"eviction": eviction, "user": user.profile.main_character.character_name},
    )

    return redirect("evictionsplit:eviction", eviction_id)


@login_required
@permission_required("evictionsplit.basic_access")
def eviction_view(request, eviction_id):
    """Displays information on an eviction"""
    logger.info("Displaying eviction id %s", eviction_id)
    eviction = get_object_or_404(Eviction, id=eviction_id)
    if not (eviction.active or request.user.has_perm("evictionsplit.manage")):
        logger.warning(
            "User id %s tried to view finished eviction id %s",
            request.user.id,
            eviction_id,
        )
        messages.warning(request, _("This eviction is now finished"))
        return redirect("evictionsplit:index")
    is_standby = eviction.is_user_on_standby(request.user)
    is_doorstop = eviction.is_user_on_doorstop(request.user)
    unknown_change = request.GET.get("unknown")

    participants = []
    for participant in eviction.participants.all():
        participants.append(
            {
                "name": participant.profile.main_character.character_name,
                "portrait_url": participant.profile.main_character.portrait_url,
                "is_standby": eviction.is_user_on_standby(participant),
                "is_doorstop": eviction.is_user_on_doorstop(participant),
            }
        )

    applicants = []
    if eviction.has_applicants() and request.user.has_perm("evictionsplit.manager"):
        for applicant in eviction.get_applicants():
            applicants.append(
                {
                    "name": applicant.profile.main_character.character_name,
                    "id": applicant.id,
                    "portrait_url": applicant.profile.main_character.portrait_url,
                }
            )

    return render(
        request,
        "evictionsplit/eviction.html",
        add_common_context(
            request,
            {
                "eviction": eviction,
                "is_standby": is_standby,
                "is_doorstop": is_doorstop,
                "time_standby": eviction.user_standby_time(request.user),
                "time_doorstop": eviction.user_doorstop_time(request.user),
                "participants": participants,
                "applicants": applicants,
                "unknown_change": unknown_change,
            },
        ),
    )


# pylint: disable=too-many-branches
@login_required
@permission_required("evictionsplit.basic_access")
@require_POST
def change_clocking(request, eviction_id):
    """Edit the clocking status of a user"""
    logger.info(
        "Changing clocking for user id %s on eviction id %s",
        request.user.id,
        eviction_id,
    )
    eviction = Eviction.objects.get(id=eviction_id)
    user = request.user

    if change := request.POST.get("change"):
        logger.debug("Change value: %s", change)
        if change == "start":
            start = True
        elif change == "stop":
            start = False
        else:
            logger.error("Change parameter value not recognized: %s", change)
            messages.error(request, _("Something went wrong"))
            return redirect("evictionsplit:index")
    else:
        logger.error("No change parameter in the request")
        messages.error(request, _("Something went wrong"))
        return redirect("evictionsplit:index")

    clocking_type = request.POST.get("clocking_type")
    if clocking_type == Clocking.DOORSTOP:
        logger.debug("Clocking_type is doorstop")
        if start:
            eviction.user_start_doorstop(user)
            messages.success(request, _("Started doorstop"))
        else:
            eviction.user_stop_doorstop(user)
            messages.success(request, _("Stopped doorstop"))
    elif clocking_type == Clocking.STANDBY_FOR_PINGS:
        logger.debug("Clocking_type is standby")
        if start:
            eviction.user_start_standby_for_ping(user)
            messages.success(request, _("Started standby for pings"))
        else:
            eviction.user_stop_standby_for_ping(user)
            messages.success(request, _("Stopped standby for pings"))
    else:
        logger.debug("Clocking_type value not recognized: %s", clocking_type)
        messages.error(request, _("Something went wrong"))
        return redirect("evictionsplit:index")

    if request.POST.get("origin") == "eviction":
        logger.debug("Returning to the eviction")
        return redirect("evictionsplit:eviction", eviction_id)

    logger.debug("Returning to index")
    return redirect("evictionsplit:index")


@login_required
@permission_required("evictionsplit.manager")
def management(request):
    """Manage evictions"""
    logger.info("Displaying management window")
    active_evictions = Eviction.objects.filter(active=True)
    inactive_evictions = Eviction.objects.filter(active=False)

    return render(
        request,
        "evictionsplit/management.html",
        add_common_context(
            request,
            {
                "active_evictions": active_evictions,
                "inactive_evictions": inactive_evictions,
            },
        ),
    )


@login_required
@permission_required("evictionsplit.manager")
def stop_eviction(request, eviction_id):
    """Stops an eviction"""
    logger.info("Stopping eviction id %s", eviction_id)
    eviction = Eviction.objects.get(id=eviction_id)
    eviction.stop()

    messages.success(request, _("Successfully stopped the eviction %s") % eviction)

    return redirect("evictionsplit:manage")



from django.contrib.auth.models import User
from django.test import TestCase
from django.utils import timezone

from ..models import Clocking, Eviction


class EvictionTest(TestCase):
    def test_create_eviction(self):
        user1 = User.objects.create_user("user1", "", "")
        Eviction.create(
            name="Test eviction",
            creator=user1,
        )

        self.assertTrue(True)

    def test_user_part_eviction(self):
        user1 = User.objects.create_user("user1", "", "")
        user2 = User.objects.create_user("user2", "", "")
        eviction = Eviction.create(
            name="Test eviction",
            creator=user1,
        )

        self.assertTrue(eviction.is_user_participating(user1))
        self.assertTrue(not eviction.is_user_participating(user2))

        eviction.participants.add(user2)

        self.assertTrue(eviction.is_user_participating(user2))

    def test_eviction_has_applicants(self):
        user1 = User.objects.create_user("user1", "", "")
        user2 = User.objects.create_user("user2", "", "")
        eviction = Eviction.create(
            name="Test eviction",
            creator=user1,
        )

        self.assertFalse(eviction.has_applicants())

        eviction.user_apply(user2)

        self.assertTrue(eviction.has_applicants())

        eviction.validate_apply(user2)

        self.assertFalse(eviction.has_applicants())

    def test_eviction_application(self):
        user1 = User.objects.create_user("user1", "", "")
        user2 = User.objects.create_user("user2", "", "")
        eviction = Eviction.create(
            name="Test eviction",
            creator=user1,
        )

        eviction.user_apply(user2)

        self.assertTrue(eviction.applicants.filter(id=user2.id).exists())
        self.assertEqual(eviction.get_applicants(), [user2])

        eviction.validate_apply(user2)

        self.assertTrue(len(eviction.applicants.all()) == 0)
        self.assertTrue(eviction.participants.filter(id=user2.id).exists())
        self.assertEqual(eviction.get_applicants(), [])

    def test_eviction_clocking_time(self):
        user1 = User.objects.create_user("user1", "", "")
        eviction = Eviction.create(
            name="Test eviction",
            creator=user1,
        )

        clock_1 = Clocking(
            user=user1,
            eviction=eviction,
            type=Clocking.DOORSTOP,
            beginning=timezone.datetime(2023, 1, 1, 12, 0, 0, 0),
            end=timezone.datetime(2023, 1, 1, 12, 10, 0, 0),
        )
        clock_1.save()

        self.assertEquals(
            timezone.timedelta(minutes=10), eviction.user_doorstop_time(user1)
        )

        clock_2 = Clocking(
            user=user1,
            eviction=eviction,
            type=Clocking.DOORSTOP,
            beginning=timezone.datetime(2023, 1, 1, 13, 0, 0, 0),
            end=timezone.datetime(2023, 1, 1, 13, 20, 0, 0),
        )
        clock_2.save()

        self.assertEquals(
            timezone.timedelta(minutes=30), eviction.user_doorstop_time(user1)
        )

        clock_3 = Clocking(
            user=user1,
            eviction=eviction,
            type=Clocking.STANDBY_FOR_PINGS,
            beginning=timezone.datetime(2023, 1, 1, 12, 0, 0, 0),
            end=timezone.datetime(2023, 1, 1, 12, 10, 0, 0),
        )
        clock_3.save()

        self.assertEquals(
            timezone.timedelta(minutes=30), eviction.user_doorstop_time(user1)
        )
        self.assertEquals(
            timezone.timedelta(minutes=10), eviction.user_standby_time(user1)
        )

    def test_eviction_stop(self):
        user1 = User.objects.create_user("user1", "", "")
        eviction = Eviction.create(
            name="Test eviction",
            creator=user1,
        )

        clock_1 = Clocking(
            user=user1,
            eviction=eviction,
            type=Clocking.DOORSTOP,
            beginning=timezone.datetime(2023, 1, 1, 12, 0, 0, 0),
        )
        clock_1.save()

        clock_2 = Clocking(
            user=user1,
            eviction=eviction,
            type=Clocking.STANDBY_FOR_PINGS,
            beginning=timezone.datetime(2023, 1, 1, 12, 0, 0, 0),
        )
        clock_2.save()

        clock_3 = Clocking(
            user=user1,
            eviction=eviction,
            type=Clocking.STANDBY_FOR_PINGS,
            beginning=timezone.datetime(2023, 1, 1, 11, 0, 0, 0),
            end=timezone.datetime(2023, 1, 1, 11, 10, 0, 0),
        )
        clock_3.save()

        self.assertTrue(eviction.active)
        self.assertIsNone(clock_1.end)
        self.assertIsNone(clock_2.end)

        eviction.stop()

        self.assertFalse(eviction.active)
        self.assertIsNotNone(Clocking.objects.get(id=clock_1.id).end)
        self.assertIsNotNone(Clocking.objects.get(id=clock_2.id).end)
        self.assertIsNotNone(Clocking.objects.get(id=clock_3.id).end)

    def test_cant_clock_inactive(self):
        user1 = User.objects.create_user("user1", "", "")
        eviction = Eviction.create(
            name="Test eviction",
            creator=user1,
        )
        eviction.stop()

        with self.assertRaises(Eviction.EvictionInactive):
            eviction.user_start_doorstop(user1)

        with self.assertRaises(Eviction.EvictionInactive):
            eviction.user_start_standby_for_ping(user1)

    def test_eviction_export_activity(self):
        user1 = User.objects.create_user("user1", "", "")
        eviction = Eviction.create(
            name="Test eviction",
            creator=user1,
        )

        clock_1 = Clocking(
            user=user1,
            eviction=eviction,
            type=Clocking.DOORSTOP,
            beginning=timezone.datetime(2023, 1, 1, 12, 0, 0, 0),
            end=timezone.datetime(2023, 1, 1, 15, 20, 30, 0),
        )
        clock_1.save()

        clock_2 = Clocking(
            user=user1,
            eviction=eviction,
            type=Clocking.STANDBY_FOR_PINGS,
            beginning=timezone.datetime(2023, 1, 1, 12, 0, 0, 0),
            end=timezone.datetime(2023, 1, 2, 13, 30, 20),
        )
        clock_2.save()

        clock_3 = Clocking(
            user=user1,
            eviction=eviction,
            type=Clocking.STANDBY_FOR_PINGS,
            beginning=timezone.datetime(2023, 1, 1, 11, 0, 0, 0),
            end=timezone.datetime(2023, 1, 1, 11, 10, 0, 0),
        )
        clock_3.save()
        eviction.stop()

        activity_export = eviction.spreadsheet_export()
        expected = """PILOT\tSTANDBY\tDOORSTOP
user1\t25:40:20\t03:20:30\n"""

        self.assertEquals(expected, activity_export)

        user2 = User.objects.create_user("user2", "", "")
        eviction.participants.add(user2)

        clock_4 = Clocking(
            user=user2,
            eviction=eviction,
            type=Clocking.STANDBY_FOR_PINGS,
            beginning=timezone.datetime(2023, 1, 1, 12, 0, 0, 0),
            end=timezone.datetime(2023, 1, 1, 13, 10, 0, 0),
        )
        clock_4.save()

        clock_5 = Clocking(
            user=user2,
            eviction=eviction,
            type=Clocking.DOORSTOP,
            beginning=timezone.datetime(2023, 1, 1, 12, 20, 0, 0),
            end=timezone.datetime(2023, 1, 1, 12, 30, 0, 0),
        )
        clock_5.save()

        activity_export = eviction.spreadsheet_export()
        expected = """PILOT\tSTANDBY\tDOORSTOP
user1\t25:40:20\t03:20:30
user2\t01:10:00\t00:10:00\n"""

        self.assertEqual(expected, activity_export)



"""https://stackoverflow.com/a/46928226"""

from django import template

register = template.Library()


@register.filter()
def smooth_timedelta(timedeltaobj):
    """Convert a datetime.timedelta object into Days, Hours, Minutes, Seconds."""
    secs = timedeltaobj.total_seconds()
    total_time = ""
    if secs > 86400:  # 60sec * 60min * 24hrs
        days = secs // 86400
        total_time += f"{int(days)} days"
        secs = secs - days * 86400

    if secs > 3600:
        hrs = secs // 3600
        total_time += f" {int(hrs)} hours"
        secs = secs - hrs * 3600

    if secs > 60:
        mins = secs // 60
        total_time += f" {int(mins)} minutes"
        secs = secs - mins * 60

    if secs > 0:
        total_time += f" {int(secs)} seconds"
    return total_time



"""Models."""
import datetime
from typing import List

from django.db import models
from django.utils import timezone

from allianceauth.authentication.models import User, UserProfile


class General(models.Model):
    """Meta model for evictionsplit permissions"""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ("basic_access", "Can access this app"),
            ("manager", "Can create and manage evictions"),
        )


def format_delta_for_export(delta: datetime.timedelta) -> str:
    """
    Returns a timedelta formatted as follows: HOUR:MINUTES:SECONDS
    Will not display the number of days and will add 24 hours instead
    """

    remaining_value = delta.seconds
    seconds = remaining_value % 60
    remaining_value = remaining_value // 60
    minutes = remaining_value % 60
    hours = (remaining_value // 60) + delta.days * 24

    return f"{hours:02d}:{minutes:02d}:{seconds:02d}"


class Eviction(models.Model):
    """Model for an eviction people can subscribe to"""

    name = models.CharField(max_length=120)
    creator = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        related_name="evictions_created",
        related_query_name="eviction_created",
    )
    participants = models.ManyToManyField(
        User,
        related_name="evictions_participating",
        related_query_name="eviction_participating",
        blank=True,
    )
    applicants = models.ManyToManyField(
        User, related_name="evictions_applied", related_query_name="eviction_applied"
    )
    active = models.BooleanField(default=True)

    class EvictionInactive(Exception):
        """Signals that the eviction is now inactive"""

    @classmethod
    def create(cls, name: str, creator: User) -> "Eviction":
        """Creates and eviction in the databaser and returns it"""
        e = Eviction.objects.create(name=name, creator=creator)
        e.participants.add(creator)
        e.save()
        return e

    def __str__(self):
        return self.name

    def __repr__(self):
        profile: UserProfile = self.creator.profile  # pylint: disable=no-member
        return f"{self.name} by {profile.main_character.character_name}"

    def is_user_participating(self, user: User) -> bool:
        """True if the user is an eviction participant"""
        return self.participants.filter(id=user.id).exists()

    def user_apply(self, user: User):
        """Adds a user to the list of applicants if they are not already part of the eviction"""
        if not (
            self.participants.filter(id=user.id).exists()
            or self.applicants.filter(id=user.id).exists()
        ):
            self.applicants.add(user)

    def validate_apply(self, user: User):
        """Remove a user from the applicants and add them as a participant"""
        self.applicants.remove(user)
        self.participants.add(user)

    def has_applicants(self) -> bool:
        """True if the eviction has users with applications waiting"""
        return self.applicants.all().exists()

    def get_applicants(self) -> List[User]:
        """List of users with applications waiting"""
        return list(self.applicants.all())

    def is_user_on_standby(self, user: User) -> bool:
        """True if the user is currently on standby for ping"""
        return self.__is_user_clocked(user, Clocking.STANDBY_FOR_PINGS)

    def is_user_on_doorstop(self, user: User) -> bool:
        """True if the user is currently on doorstop"""
        return self.__is_user_clocked(user, Clocking.DOORSTOP)

    def user_start_standby_for_ping(self, user: User):
        """Puts the user on standby for pings"""
        self.__user_start_clocking(user, Clocking.STANDBY_FOR_PINGS)

    def user_start_doorstop(self, user: User):
        """Puts the user on doorstop"""
        self.__user_start_clocking(user, Clocking.DOORSTOP)

    def user_stop_standby_for_ping(self, user: User):
        """Removes the user from standby for pings"""
        self.__user_stop_clocking(user, Clocking.STANDBY_FOR_PINGS)

    def user_stop_doorstop(self, user: User):
        """Removes the user from doorstop"""
        self.__user_stop_clocking(user, Clocking.DOORSTOP)

    def __is_user_clocked(self, user: User, clocking_type) -> bool:
        """Abstract method to know if a user is in a specific clocking"""
        return Clocking.objects.filter(
            eviction=self, user=user, type=clocking_type, end__isnull=True
        ).exists()

    def __user_start_clocking(self, user: User, clocking_type):
        """Abstract method to start a user's clocking"""
        if not self.active:
            raise self.EvictionInactive
        if not Clocking.objects.filter(
            eviction=self, user=user, type=clocking_type, end__isnull=True
        ).exists():
            Clocking.objects.create(user=user, eviction=self, type=clocking_type)

    def __user_stop_clocking(self, user: User, clocking_type):
        """Abstract method to stop a user's clocking"""
        if clocking := Clocking.objects.get(
            eviction=self, user=user, type=clocking_type, end__isnull=True
        ):
            clocking.stop()

    def user_standby_time(self, user: User):
        """Returns for how long the user was on standby"""
        return self.__user_clocking_time(user, Clocking.STANDBY_FOR_PINGS)

    def user_doorstop_time(self, user: User):
        """Returns for how long the user was on doorstop"""
        return self.__user_clocking_time(user, Clocking.DOORSTOP)

    def __user_clocking_time(self, user: User, clocking_type):
        """Abstract method to know how long a user was on a specific duty"""
        clockings = Clocking.objects.filter(
            eviction=self, user=user, type=clocking_type
        )
        return sum(map(lambda x: x.duration, clockings), timezone.timedelta())

    def stop(self):
        """Marks eviction as inactive and stops all active clocking"""
        self.active = False
        self.save()

        active_clocking = Clocking.objects.filter(eviction=self, end__isnull=True)
        for clocking in active_clocking:
            clocking.stop()

    def spreadsheet_export(self):
        """
        Returns the time spent on standby for ping and doorstop made by each participant.
        The format is adapted to spreadsheets.

        ex:
        PILOT   STANDBY     DOORSTOP
        pilot1  02:04:00    01:30:00
        pilot2  08:25:00    03:00:00
        """

        out_str = "PILOT\tSTANDBY\tDOORSTOP\n"

        for participant in self.participants.all():
            name = (
                participant.profile.main_character.character_name
                if participant.profile.main_character
                else f"{participant.username}"
            )
            standby = format_delta_for_export(self.user_standby_time(participant))
            doorstop = format_delta_for_export(self.user_doorstop_time(participant))

            out_str += f"{name}\t{standby}\t{doorstop}\n"

        return out_str


class Clocking(models.Model):
    """Representing a user spending time on standby for ping or doorstop"""

    DOORSTOP = "DS"
    STANDBY_FOR_PINGS = "SP"

    user = models.ForeignKey(User, on_delete=models.CASCADE)
    eviction = models.ForeignKey(Eviction, on_delete=models.CASCADE)
    beginning = models.DateTimeField(default=timezone.now)
    end = models.DateTimeField(null=True)
    type = models.CharField(
        max_length=2,
        choices=[
            (DOORSTOP, "Doorstopping"),
            (STANDBY_FOR_PINGS, "Standby for ping"),
        ],
    )

    def __repr__(self):
        return f"{self.user} - {self.type} - {self.beginning}"

    def __str__(self):
        clocking_type = "Doorstop" if self.type == self.DOORSTOP else "Standby for ping"
        profile: UserProfile = self.user.profile  # pylint: disable=no-member
        return f"{clocking_type} by {profile.main_character.character_name} started at {self.beginning}"

    @property
    def duration(self):
        """Duration of this clocking"""
        if self.end is not None:
            return self.end - self.beginning

        return timezone.now() - self.beginning

    def stop(self):
        """Stops the clocking"""
        if self.end:
            return
        self.end = timezone.now()
        self.save()



from django.apps import AppConfig

from . import __version__


class EvictionSplitConfig(AppConfig):
    name = "evictionsplit"
    label = "evictionsplit"
    verbose_name = f"Eviction Split v{__version__}"
